@@ -26,6 +26,7 @@
 #include <cstdlib>
 #include <pwd.h>
 #include <string>
+#include <strings.h>
 #include <sys/types.h>
 
 #include "nethogs.h"
@@ -45,6 +46,7 @@ extern bool sortRecv;
 
 extern int viewMode;
 extern bool showcommandline;
+extern bool showBasename;
 
 extern unsigned refreshlimit;
 extern unsigned refreshcount;
@@ -67,6 +69,8 @@ const char *COLUMN_FORMAT_RECEIVED = "%11.3f";
 const char *const desc_view_mode[VIEWMODE_COUNT] = {
     "KB/sec", "KB    ", "B     ", "MB    ", "MB/sec", "GB/sec"};
 
+constexpr char FILE_SEPARATOR = '/';
+
 class Line {
 public:
   Line(const char *name, const char *cmdline, double n_recv_value,
@@ -152,6 +156,12 @@ static void mvaddstr_truncate_trailing(int row, int col, const char *str,
 static void mvaddstr_truncate_cmdline(int row, int col, const char *progname,
                                       const char *cmdline,
                                       std::size_t max_len) {
+  if (showBasename) {
+    if (index(progname, FILE_SEPARATOR) != NULL) {
+      progname = rindex(progname, FILE_SEPARATOR) + 1;
+    } 
+  }                                      
+
   std::size_t proglen = strlen(progname);
   std::size_t max_cmdlen;
 
@@ -305,6 +315,10 @@ void ui_tick() {
     /* switch mode: total vs kb/s */
     viewMode = (viewMode + 1) % VIEWMODE_COUNT;
     break;
+  case 'b':
+    /* show only the process basename */
+    showBasename = !showBasename;
+    break;
   }
 }
 
@@ -318,13 +332,10 @@ void show_trace(Line *lines[], int nproc) {
   }
 
   /* print the 'unknown' connections, for debugging */
-  ConnList *curr_unknownconn = unknowntcp->connections;
-  while (curr_unknownconn != NULL) {
+  for (auto it = unknowntcp->connections.begin(); it != unknowntcp->connections.end(); ++it) {
     std::cout << "Unknown connection: "
-              << curr_unknownconn->getVal()->refpacket->gethashstring()
+              << (*it)->refpacket->gethashstring()
               << std::endl;
-
-    curr_unknownconn = curr_unknownconn->getNext();
   }
 }
 
@@ -374,9 +385,9 @@ void show_ncurses(Line *lines[], int nproc) {
   int totalrow = std::min(rows - 1, 3 + 1 + i);
   mvprintw(totalrow, 0, "  TOTAL        %-*.*s %-*.*s    %11.3f %11.3f ",
            proglen, proglen, "", devlen, devlen, "", sent_global, recv_global);
-  mvprintw(3 + 1 + i, cols - COLUMN_WIDTH_UNIT, desc_view_mode[viewMode]);
+  mvprintw(3 + 1 + i, cols - COLUMN_WIDTH_UNIT, "%s", desc_view_mode[viewMode]);
   attroff(A_REVERSE);
-  mvprintw(totalrow + 1, 0, "");
+  mvprintw(totalrow + 1, 0, "%s", "");
   refresh();
 }
 
