@@ -1,5 +1,6 @@
 #include "nethogs.cpp"
 #include <fcntl.h>
+#include <set>
 #include <vector>
 
 #ifdef __linux__
@@ -14,6 +15,8 @@
 static std::pair<int, int> self_pipe = std::make_pair(-1, -1);
 static time_t last_refresh_time = 0;
 
+std::set<pid_t> pidsToWatch;
+
 // selectable file descriptors for the main loop
 static fd_set pc_loop_fd_set;
 static std::vector<int> pc_loop_fd_list;
@@ -26,12 +29,12 @@ static void help(bool iserror) {
 
   // output << "usage: nethogs [-V] [-b] [-d seconds] [-t] [-p] [-f (eth|ppp))]
   // [device [device [device ...]]]\n";
-  output << "usage: nethogs [-V] [-h] [-b] [-d seconds] [-v mode] [-c count] "
-            "[-t] [-p] [-s] [-a] [-l] [-f filter] [-C]"
+  output << "usage: nethogs [-V] [-h] [-x] [-d seconds] [-v mode] [-c count] "
+            "[-t] [-p] [-s] [-a] [-l] [-f filter] [-C] [-b] [-P pid] "
             "[device [device [device ...]]]\n";
   output << "		-V : prints version.\n";
   output << "		-h : prints this help.\n";
-  output << "		-b : bughunt mode - implies tracemode.\n";
+  output << "		-x : bughunt mode - implies tracemode.\n";
   output << "		-d : delay for update refresh rate in seconds. default "
             "is 1.\n";
   output << "		-v : view mode (0 = KB/s, 1 = total KB, 2 = total B, 3 "
@@ -46,17 +49,22 @@ static void help(bool iserror) {
   output << "		-a : monitor all devices, even loopback/stopped "
             "ones.\n";
   output << "		-C : capture TCP and UDP.\n";
+  output << "		-g : garbage collection period in number of refresh. "
+            "default is 50.\n";
+  output << "		-b : Short program name. Displays only the program name.\n";
   output << "		-f : EXPERIMENTAL: specify string pcap filter (like "
             "tcpdump)."
             " This may be removed or changed in a future version.\n";
   output << "		device : device(s) to monitor. default is all "
             "interfaces up and running excluding loopback\n";
+  output << "		-P : Show only processes.\n";
   output << std::endl;
   output << "When nethogs is running, press:\n";
   output << " q: quit\n";
   output << " s: sort by SENT traffic\n";
-  output << " r: sort by RECEIVE traffic\n";
+  output << " r: sort by RECEIVED traffic\n";
   output << " l: display command line\n";
+  output << " b: display the program basename instead of the fullpath\n";
   output << " m: switch between total (KB, B, MB) and throughput (KB/s, MB/s, "
             "GB/s) mode\n";
 }
@@ -140,9 +148,11 @@ int main(int argc, char **argv) {
   int promisc = 0;
   bool all = false;
   char *filter = NULL;
+  int garbage_collection_period = 50;
+
 
   int opt;
-  while ((opt = getopt(argc, argv, "Vhbtpsd:v:c:laf:C")) != -1) {
+  while ((opt = getopt(argc, argv, "Vhxtpsd:v:c:laf:Cbg:P:")) != -1) {
     switch (opt) {
     case 'V':
       versiondisplay();
@@ -150,7 +160,7 @@ int main(int argc, char **argv) {
     case 'h':
       help(false);
       exit(0);
-    case 'b':
+    case 'x':
       bughuntmode = true;
       tracemode = true;
       break;
@@ -184,6 +194,15 @@ int main(int argc, char **argv) {
     case 'C':
       catchall = true;
       break;
+    case 'b':
+      showBasename = true;
+      break;
+    case 'g':
+      garbage_collection_period = (time_t)atoi(optarg);
+      break;
+    case 'P':
+      pidsToWatch.insert((pid_t) atoi(optarg));
+      break;
     default:
       help(true);
       exit(EXIT_FAILURE);
@@ -200,27 +219,11 @@ int main(int argc, char **argv) {
     init_ui();
   }
 
-  if (geteuid() != 0) {
-#ifdef __linux__
-    char exe_path[PATH_MAX];
-    ssize_t len;
-    unsigned int caps[5] = {0, 0, 0, 0, 0};
-
-    if ((len = readlink("/proc/self/exe", exe_path, PATH_MAX)) == -1)
-      forceExit(false, "Failed to locate nethogs binary.");
-    exe_path[len] = '\0';
-
-    getxattr(exe_path, "security.capability", (char *)caps, sizeof(caps));
-
-    if ((((caps[1] >> CAP_NET_ADMIN) & 1) != 1) ||
-        (((caps[1] >> CAP_NET_RAW) & 1) != 1))
-      forceExit(false, "To run nethogs without being root you need to enable "
-                       "capabilities on the program (cap_net_admin, "
-                       "cap_net_raw), see the documentation for details.");
-#else
+#ifndef __linux__
+  if (geteuid() != 0)
     forceExit(false, "You need to be root to run NetHogs!");
 #endif
-  }
+  // on Linux, we can run as non-root given the cap_net_admin and cap_net_raw capabilities
 
   // use the Self-Pipe trick to interrupt the select() in the main loop
   self_pipe = create_self_pipe();
@@ -286,6 +289,10 @@ int main(int argc, char **argv) {
   }
 
   if (nb_devices == nb_failed_devices) {
+    if (geteuid() != 0)
+      fprintf(stderr, "To run nethogs without being root, you need to enable "
+                      "capabilities on the program (cap_net_admin, cap_new_raw). "
+                      "See the documentation for details.\n");
     forceExit(false, "Error opening pcap handlers for all devices.\n");
   }
 
@@ -294,6 +301,7 @@ int main(int argc, char **argv) {
   struct dpargs *userdata = (dpargs *)malloc(sizeof(struct dpargs));
 
   // Main loop:
+  int refresh_count = 0;
   while (1) {
     bool packets_read = false;
 
@@ -322,6 +330,12 @@ int main(int argc, char **argv) {
         ui_tick();
       }
       do_refresh();
+      ++refresh_count;
+
+      if ((garbage_collection_period > 0) &&
+          (refresh_count % garbage_collection_period == 0)) {
+        garbage_collect_processes();
+      }
     }
 
     // if not packets, do a select() until next packet
