@@ -22,6 +22,7 @@
 
 #include <iostream>
 #include <ncurses.h>
+#include <set>
 #include <string>
 #include <strings.h>
 #if !defined(__APPLE__) && !defined(__FreeBSD__)
@@ -34,6 +35,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <signal.h>
 
 #include "conninode.h"
 #include "inode2prog.h"
@@ -47,7 +49,8 @@ extern bool catchall;
  * key contains source ip, source port, destination ip, destination
  * port in format: '1.2.3.4:5-1.2.3.4:5'
  */
-extern std::map<std::string, unsigned long> conninode;
+extern std::map<std::string, unsigned long> conninode_tcp;
+extern std::map<std::string, unsigned long> conninode_udp;
 
 /* this file includes:
  * - calls to inodeproc to get the pid that belongs to that inode
@@ -65,6 +68,8 @@ Process *unknownudp;
 Process *unknownip;
 ProcList *processes;
 
+extern std::set<pid_t> pidsToWatch;
+
 #define KB (1UL << 10)
 #define MB (1UL << 20)
 #define GB (1UL << 30)
@@ -76,6 +81,7 @@ float tokbps(u_int64_t bytes) { return (((double)bytes) / PERIOD) / KB; }
 float tombps(u_int64_t bytes) { return (((double)bytes) / PERIOD) / MB; }
 float togbps(u_int64_t bytes) { return (((double)bytes) / PERIOD) / GB; }
 
+
 void process_init() {
   unknowntcp = new Process(0, "", "unknown TCP");
   processes = new ProcList(unknowntcp, NULL);
@@ -86,17 +92,15 @@ void process_init() {
     // unknownip = new Process (0, "", "unknown IP");
     // processes = new ProcList (unknownip, processes);
   }
+
 }
 
 int Process::getLastPacket() {
   int lastpacket = 0;
-  ConnList *curconn = connections;
-  while (curconn != NULL) {
-    assert(curconn != NULL);
-    assert(curconn->getVal() != NULL);
-    if (curconn->getVal()->getLastPacket() > lastpacket)
-      lastpacket = curconn->getVal()->getLastPacket();
-    curconn = curconn->getNext();
+  for (auto it = connections.begin(); it != connections.end(); ++it) {
+    assert(*it != NULL);
+    if ((*it)->getLastPacket() > lastpacket)
+      lastpacket = (*it)->getLastPacket();
   }
   return lastpacket;
 }
@@ -106,30 +110,20 @@ static void sum_active_connections(Process *process_ptr, u_int64_t &sum_sent,
                                    u_int64_t &sum_recv) {
   /* walk though all process_ptr process's connections, and sum
    * them up */
-  ConnList *curconn = process_ptr->connections;
-  ConnList *previous = NULL;
-  while (curconn != NULL) {
-    if (curconn->getVal()->getLastPacket() <= curtime.tv_sec - CONNTIMEOUT) {
+  for (auto it = process_ptr->connections.begin(); it != process_ptr->connections.end(); ) {
+    if ((*it)->getLastPacket() <= curtime.tv_sec - CONNTIMEOUT) {
       /* capture sent and received totals before deleting */
-      process_ptr->sent_by_closed_bytes += curconn->getVal()->sumSent;
-      process_ptr->rcvd_by_closed_bytes += curconn->getVal()->sumRecv;
+      process_ptr->sent_by_closed_bytes += (*it)->sumSent;
+      process_ptr->rcvd_by_closed_bytes += (*it)->sumRecv;
       /* stalled connection, remove. */
-      ConnList *todelete = curconn;
-      Connection *conn_todelete = curconn->getVal();
-      curconn = curconn->getNext();
-      if (todelete == process_ptr->connections)
-        process_ptr->connections = curconn;
-      if (previous != NULL)
-        previous->setNext(curconn);
-      delete (todelete);
-      delete (conn_todelete);
+      delete (*it);
+      it = process_ptr->connections.erase(it);
     } else {
       u_int64_t sent = 0, recv = 0;
-      curconn->getVal()->sumanddel(curtime, &recv, &sent);
+      (*it)->sumanddel(curtime, &recv, &sent);
       sum_sent += sent;
       sum_recv += recv;
-      previous = curconn;
-      curconn = curconn->getNext();
+      ++it;
     }
   }
 }
@@ -164,12 +158,10 @@ void Process::getgbps(float *recvd, float *sent) {
 /** get total values for this process */
 void Process::gettotal(u_int64_t *recvd, u_int64_t *sent) {
   u_int64_t sum_sent = 0, sum_recv = 0;
-  ConnList *curconn = this->connections;
-  while (curconn != NULL) {
-    Connection *conn = curconn->getVal();
+  for (auto it = this->connections.begin(); it != this->connections.end(); ++it) {
+    Connection *conn = (*it);
     sum_sent += conn->sumSent;
     sum_recv += conn->sumRecv;
-    curconn = curconn->getNext();
   }
   // std::cout << "Sum sent: " << sum_sent << std::endl;
   // std::cout << "Sum recv: " << sum_recv << std::endl;
@@ -261,6 +253,10 @@ Process *getProcess(unsigned long inode, const char *devicename) {
   if (proc != NULL)
     return proc;
 
+  if ( !(pidsToWatch.empty()) && pidsToWatch.find(node->pid) == pidsToWatch.end() ) {
+    return NULL;
+  }
+
   // extract program name and command line from data read from cmdline file
   const char *prgname = node->cmdline.c_str();
   const char *cmdline = prgname + strlen(prgname) + 1;
@@ -306,7 +302,10 @@ Process *getProcess(unsigned long inode, const char *devicename) {
  * is made. If no process can be found even then, it's added to the
  * 'unknown' process.
  */
-Process *getProcess(Connection *connection, const char *devicename) {
+Process *getProcess(Connection *connection, const char *devicename,
+                    short int packettype) {
+  std::map<std::string, unsigned long> &conninode =
+      (packettype == IPPROTO_TCP) ? conninode_tcp : conninode_udp;
   unsigned long inode = conninode[connection->refpacket->gethashstring()];
 
   if (inode == 0) {
@@ -370,15 +369,26 @@ Process *getProcess(Connection *connection, const char *devicename) {
   }
 
   Process *proc = NULL;
-  if (inode != 0)
+  if (inode != 0) {
     proc = getProcess(inode, devicename);
+  } else {
+    if (packettype == IPPROTO_TCP) {
+      proc = unknowntcp;
+    } else {
+      proc = unknownudp;
+    }
+  }
+
+  if (!(pidsToWatch.empty()) && proc == NULL) {
+    proc = (packettype == IPPROTO_TCP) ? unknowntcp : unknownudp;
+  }
 
   if (proc == NULL) {
     proc = new Process(inode, "", connection->refpacket->gethashstring());
     processes = new ProcList(proc, processes);
   }
 
-  proc->connections = new ConnList(connection, proc->connections);
+  proc->connections.insert(connection);
   return proc;
 }
 
@@ -413,3 +423,5 @@ void remove_timed_out_processes() {
     previousproc = curproc;
   }
 }
+
+void garbage_collect_processes() { garbage_collect_inodeproc(); }
